//############################################################################
// ~ International Laser Display Association ~
// ~     ILDA File Format Driver in C++      ~
// ~             Built for EZFB              ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//############################################################################
#ifndef __EZ_WAVE_DEFINITIONS__
#define __EZ_WAVE_DEFINITIONS__

//############################################################################
#include "ez_ilda_common.hpp"

//############################################################################
class ez_wave_optimization_parameters
{
public:
    ez_wave_optimization_parameters()
              : lit_dwell_overhang     (0),
                distance_delta_max     (0),
                longest_dwell_microsec (0),
                insignificant_distance (0),
                zero_angle_tolerance   (0),
                frames_per_second      (0)
                                            {};
   ~ez_wave_optimization_parameters()       {};
    //------------------------------------------------------------------------
    int    lit_dwell_overhang;
    float  distance_delta_max,
           longest_dwell_microsec,
           insignificant_distance,
           zero_angle_tolerance,
           frames_per_second;
};

//############################################################################
class ez_wave_header
{
public:
    ez_wave_header( const int    s = 0,     // num_samples
                    const int    r = 48000, // rate
                    const short  n = 6,     // channels
                    const short  b = 16     // bits_per_sample
                  );
    //------------------------------------------------------------------------
   ~ez_wave_header();
    //------------------------------------------------------------------------
    ez_wave_header(const ez_wave_header& header);
    //------------------------------------------------------------------------
    ez_wave_header& operator = (const ez_wave_header& header);
    //------------------------------------------------------------------------
    int      from_ifstream_wave (ifstream& in);
    //------------------------------------------------------------------------
    void     to_ofstream_wave   (ofstream& out,
                                 bool with_laserboy_stuff = true
                                ) const ;
    //------------------------------------------------------------------------
    int      num_samples     ,
             sample_rate     ,
            *offset          ;             
    //------------------------------------------------------------------------
    u_int    ez_wave_mode    ; // up to 32 binary flags
    //------------------------------------------------------------------------
    short    num_channels    ,
             bits_per_sample ;
    //------------------------------------------------------------------------
    string   header_version  ;
    //------------------------------------------------------------------------
    ez_wave_optimization_parameters parms;
};

//############################################################################
class ez_wave_sample
{
public:
    ez_wave_sample(int n = 6)
        {
            num_channels = n;
            channel = new short[num_channels];
            memset((void*)channel, 0x00, num_channels * sizeof(short));
        }
    //------------------------------------------------------------------------
   ~ez_wave_sample()    {    delete channel;    }
    //------------------------------------------------------------------------
    bool from_ifstream_wave (ifstream& in)
            {
                if(!in.eof())
                {
                    in.read((char*)channel, num_channels * sizeof(signed short));
                    return true;
                }
                return false; // returns false if EOF
            }
    //------------------------------------------------------------------------
    bool from_ifstream_wave_inverted(ifstream& in)
            {
                if(!in.eof())
                {
                    in.read((char*)channel, num_channels * sizeof(signed short));
                    negate();
                    return true;
                }
                return false; // returns false if EOF
            }
    //------------------------------------------------------------------------
    void to_ofstream_wave(ofstream& out)
            {
                out.write((char*)channel, num_channels * sizeof(signed short));
                return;
            }
    //------------------------------------------------------------------------
    void to_ofstream_wave_inverted(ofstream& out)
            {
                negate();
                out.write((char*)channel, num_channels * sizeof(signed short));
                negate();
                return;
            }
    //------------------------------------------------------------------------
    ez_wave_sample& operator = (const ez_wave_sample& sample)
                    {
                        delete channel;
                        num_channels = sample.num_channels;
                        channel = new short[num_channels];
                        memcpy((void*)channel, (void*)sample.channel, num_channels * sizeof(short));
                        return *this;
                    }
    //------------------------------------------------------------------------
    void negate()
            {
                for(int i = 0; i < num_channels; i++)
                    channel[i] = -channel[i];
                return;
            }
    //------------------------------------------------------------------------
    ez_wave_sample  operator + (const ez_wave_sample& s) // side-by-side as tracks
                    {
                        ez_wave_sample sum(num_channels + s.num_channels);
                        memcpy(   (void*)sum.channel,
                                  (void*)channel,
                                  num_channels * sizeof(short)
                              );                              
                        memcpy(   (void*)(sum.channel + num_channels), // + offset for above
                                  (void*)s.channel,
                                  s.num_channels * sizeof(short)
                              );
                        return sum;
                    }
    //------------------------------------------------------------------------
    signed short   *channel     ;
    int             num_channels;
};

//############################################################################
typedef ez_wave_sample*  ez_wave_sample_pointer;

//############################################################################
#endif

//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################
